@@ -1,7 +1,7 @@
-//  Copyright (c) 2014, Facebook, Inc.  All rights reserved.
-//  This source code is licensed under the BSD-style license found in the
-//  LICENSE file in the root directory of this source tree. An additional grant
-//  of patent rights can be found in the PATENTS file in the same directory.
+//  Copyright (c) 2011-present, Facebook, Inc.  All rights reserved.
+//  This source code is licensed under both the GPLv2 (found in the
+//  COPYING file in the root directory) and Apache 2.0 License
+//  (found in the LICENSE.Apache file in the root directory).
 
 #ifndef ROCKSDB_LITE
 #include "table/cuckoo_table_builder.h"
@@ -52,7 +52,8 @@ CuckooTableBuilder::CuckooTableBuilder(
     uint32_t max_num_hash_table, uint32_t max_search_depth,
     const Comparator* user_comparator, uint32_t cuckoo_block_size,
     bool use_module_hash, bool identity_as_first_hash,
-    uint64_t (*get_slice_hash)(const Slice&, uint32_t, uint64_t))
+    uint64_t (*get_slice_hash)(const Slice&, uint32_t, uint64_t),
+    uint32_t column_family_id, const std::string& column_family_name)
     : num_hash_func_(2),
       file_(file),
       max_hash_table_ratio_(max_hash_table_ratio),
@@ -76,6 +77,8 @@ CuckooTableBuilder::CuckooTableBuilder(
   properties_.num_data_blocks = 1;
   properties_.index_size = 0;
   properties_.filter_size = 0;
+  properties_.column_family_id = column_family_id;
+  properties_.column_family_name = column_family_name;
 }
 
 void CuckooTableBuilder::Add(const Slice& key, const Slice& value) {
@@ -109,8 +112,6 @@ void CuckooTableBuilder::Add(const Slice& key, const Slice& value) {
     status_ = Status::NotSupported("all keys have to be the same size");
     return;
   }
-  // Even if one sequence number is non-zero, then it is not last level.
-  assert(!is_last_level_file_ || ikey.sequence == 0);
 
   if (ikey.type == kTypeValue) {
     if (!has_seen_first_value_) {
@@ -248,7 +249,8 @@ Status CuckooTableBuilder::Finish() {
   if (num_entries_ > 0) {
     // Calculate the real hash size if module hash is enabled.
     if (use_module_hash_) {
-      hash_table_size_ = num_entries_ / max_hash_table_ratio_;
+      hash_table_size_ =
+        static_cast<uint64_t>(num_entries_ / max_hash_table_ratio_);
     }
     s = MakeHashTable(&buckets);
     if (!s.ok()) {
@@ -404,7 +406,8 @@ uint64_t CuckooTableBuilder::FileSize() const {
   }
 
   if (use_module_hash_) {
-    return (key_size_ + value_size_) * num_entries_ / max_hash_table_ratio_;
+    return static_cast<uint64_t>((key_size_ + value_size_) *
+        num_entries_ / max_hash_table_ratio_);
   } else {
     // Account for buckets being a power of two.
     // As elements are added, file size remains constant for a while and
