@@ -1,7 +1,7 @@
-//  Copyright (c) 2013, Facebook, Inc.  All rights reserved.
-//  This source code is licensed under the BSD-style license found in the
-//  LICENSE file in the root directory of this source tree. An additional grant
-//  of patent rights can be found in the PATENTS file in the same directory.
+//  Copyright (c) 2011-present, Facebook, Inc.  All rights reserved.
+//  This source code is licensed under both the GPLv2 (found in the
+//  COPYING file in the root directory) and Apache 2.0 License
+//  (found in the LICENSE.Apache file in the root directory).
 
 #ifndef ROCKSDB_LITE
 #include "table/plain_table_key_coding.h"
@@ -112,10 +112,10 @@ Status PlainTableKeyEncoder::AppendKey(const Slice& key,
 
     Slice prefix =
         prefix_extractor_->Transform(Slice(key.data(), user_key_size));
-    if (key_count_for_prefix_ == 0 || prefix != pre_prefix_.GetKey() ||
+    if (key_count_for_prefix_ == 0 || prefix != pre_prefix_.GetUserKey() ||
         key_count_for_prefix_ % index_sparseness_ == 0) {
       key_count_for_prefix_ = 1;
-      pre_prefix_.SetKey(prefix);
+      pre_prefix_.SetUserKey(prefix);
       size_bytes_pos += EncodeSize(kFullKey, user_key_size, size_bytes);
       Status s = file->Append(Slice(size_bytes, size_bytes_pos));
       if (!s.ok()) {
@@ -128,10 +128,11 @@ Status PlainTableKeyEncoder::AppendKey(const Slice& key,
         // For second key within a prefix, need to encode prefix length
         size_bytes_pos +=
             EncodeSize(kPrefixFromPreviousKey,
-                       static_cast<uint32_t>(pre_prefix_.GetKey().size()),
+                       static_cast<uint32_t>(pre_prefix_.GetUserKey().size()),
                        size_bytes + size_bytes_pos);
       }
-      uint32_t prefix_len = static_cast<uint32_t>(pre_prefix_.GetKey().size());
+      uint32_t prefix_len =
+          static_cast<uint32_t>(pre_prefix_.GetUserKey().size());
       size_bytes_pos += EncodeSize(kKeySuffix, user_key_size - prefix_len,
                                    size_bytes + size_bytes_pos);
       Status s = file->Append(Slice(size_bytes, size_bytes_pos));
@@ -164,47 +165,62 @@ Status PlainTableKeyEncoder::AppendKey(const Slice& key,
   return Status::OK();
 }
 
-inline bool PlainTableKeyDecoder::FileReader::Read(uint32_t file_offset,
-                                                   uint32_t len, Slice* out) {
-  if (file_info_->is_mmap_mode) {
-    assert(file_offset + len <= file_info_->data_end_offset);
-    *out = Slice(file_info_->file_data.data() + file_offset, len);
-    return true;
-  } else {
-    return ReadNonMmap(file_offset, len, out);
-  }
+Slice PlainTableFileReader::GetFromBuffer(Buffer* buffer, uint32_t file_offset,
+                                          uint32_t len) {
+  assert(file_offset + len <= file_info_->data_end_offset);
+  return Slice(buffer->buf.get() + (file_offset - buffer->buf_start_offset),
+               len);
 }
 
-bool PlainTableKeyDecoder::FileReader::ReadNonMmap(uint32_t file_offset,
-                                                   uint32_t len, Slice* out) {
+bool PlainTableFileReader::ReadNonMmap(uint32_t file_offset, uint32_t len,
+                                       Slice* out) {
   const uint32_t kPrefetchSize = 256u;
-  if (file_offset < buf_start_offset_ ||
-      file_offset + len > buf_start_offset_ + buf_len_) {
-    // Load buffer
-    assert(file_offset + len <= file_info_->data_end_offset);
-    uint32_t size_to_read = std::min(file_info_->data_end_offset - file_offset,
-                                     std::max(kPrefetchSize, len));
-    if (size_to_read > buf_capacity_) {
-      buf_.reset(new char[size_to_read]);
-      buf_capacity_ = size_to_read;
-      buf_len_ = 0;
+
+  // Try to read from buffers.
+  for (uint32_t i = 0; i < num_buf_; i++) {
+    Buffer* buffer = buffers_[num_buf_ - 1 - i].get();
+    if (file_offset >= buffer->buf_start_offset &&
+        file_offset + len <= buffer->buf_start_offset + buffer->buf_len) {
+      *out = GetFromBuffer(buffer, file_offset, len);
+      return true;
     }
-    Slice read_result;
-    Status s = file_info_->file->Read(file_offset, size_to_read, &read_result,
-                                      buf_.get());
-    if (!s.ok()) {
-      status_ = s;
-      return false;
-    }
-    buf_start_offset_ = file_offset;
-    buf_len_ = size_to_read;
   }
-  *out = Slice(buf_.get() + (file_offset - buf_start_offset_), len);
+
+  Buffer* new_buffer;
+  // Data needed is not in any of the buffer. Allocate a new buffer.
+  if (num_buf_ < buffers_.size()) {
+    // Add a new buffer
+    new_buffer = new Buffer();
+    buffers_[num_buf_++].reset(new_buffer);
+  } else {
+    // Now simply replace the last buffer. Can improve the placement policy
+    // if needed.
+    new_buffer = buffers_[num_buf_ - 1].get();
+  }
+
+  assert(file_offset + len <= file_info_->data_end_offset);
+  uint32_t size_to_read = std::min(file_info_->data_end_offset - file_offset,
+                                   std::max(kPrefetchSize, len));
+  if (size_to_read > new_buffer->buf_capacity) {
+    new_buffer->buf.reset(new char[size_to_read]);
+    new_buffer->buf_capacity = size_to_read;
+    new_buffer->buf_len = 0;
+  }
+  Slice read_result;
+  Status s = file_info_->file->Read(file_offset, size_to_read, &read_result,
+                                    new_buffer->buf.get());
+  if (!s.ok()) {
+    status_ = s;
+    return false;
+  }
+  new_buffer->buf_start_offset = file_offset;
+  new_buffer->buf_len = size_to_read;
+  *out = GetFromBuffer(new_buffer, file_offset, len);
   return true;
 }
 
-inline bool PlainTableKeyDecoder::FileReader::ReadVarint32(
-    uint32_t offset, uint32_t* out, uint32_t* bytes_read) {
+inline bool PlainTableFileReader::ReadVarint32(uint32_t offset, uint32_t* out,
+                                               uint32_t* bytes_read) {
   if (file_info_->is_mmap_mode) {
     const char* start = file_info_->file_data.data() + offset;
     const char* limit =
@@ -218,8 +234,8 @@ inline bool PlainTableKeyDecoder::FileReader::ReadVarint32(
   }
 }
 
-bool PlainTableKeyDecoder::FileReader::ReadVarint32NonMmap(
-    uint32_t offset, uint32_t* out, uint32_t* bytes_read) {
+bool PlainTableFileReader::ReadVarint32NonMmap(uint32_t offset, uint32_t* out,
+                                               uint32_t* bytes_read) {
   const char* start;
   const char* limit;
   const uint32_t kMaxVarInt32Size = 6u;
@@ -298,11 +314,12 @@ Status PlainTableKeyDecoder::NextPlainEncodingKey(uint32_t start_offset,
   if (!s.ok()) {
     return s;
   }
-  if (!file_reader_.file_info_->is_mmap_mode) {
+  if (!file_reader_.file_info()->is_mmap_mode) {
     cur_key_.SetInternalKey(*parsed_key);
-    parsed_key->user_key = Slice(cur_key_.GetKey().data(), user_key_size);
+    parsed_key->user_key =
+        Slice(cur_key_.GetInternalKey().data(), user_key_size);
     if (internal_key != nullptr) {
-      *internal_key = cur_key_.GetKey();
+      *internal_key = cur_key_.GetInternalKey();
     }
   } else if (internal_key != nullptr) {
     if (decoded_internal_key_valid) {
@@ -310,7 +327,7 @@ Status PlainTableKeyDecoder::NextPlainEncodingKey(uint32_t start_offset,
     } else {
       // Need to copy out the internal key
       cur_key_.SetInternalKey(*parsed_key);
-      *internal_key = cur_key_.GetKey();
+      *internal_key = cur_key_.GetInternalKey();
     }
   }
   return Status::OK();
@@ -348,18 +365,19 @@ Status PlainTableKeyDecoder::NextPrefixEncodingKey(
         if (!s.ok()) {
           return s;
         }
-        if (!file_reader_.file_info_->is_mmap_mode ||
+        if (!file_reader_.file_info()->is_mmap_mode ||
             (internal_key != nullptr && !decoded_internal_key_valid)) {
           // In non-mmap mode, always need to make a copy of keys returned to
           // users, because after reading value for the key, the key might
           // be invalid.
           cur_key_.SetInternalKey(*parsed_key);
-          saved_user_key_ = cur_key_.GetKey();
-          if (!file_reader_.file_info_->is_mmap_mode) {
-            parsed_key->user_key = Slice(cur_key_.GetKey().data(), size);
+          saved_user_key_ = cur_key_.GetUserKey();
+          if (!file_reader_.file_info()->is_mmap_mode) {
+            parsed_key->user_key =
+                Slice(cur_key_.GetInternalKey().data(), size);
           }
           if (internal_key != nullptr) {
-            *internal_key = cur_key_.GetKey();
+            *internal_key = cur_key_.GetInternalKey();
           }
         } else {
           if (internal_key != nullptr) {
@@ -394,7 +412,7 @@ Status PlainTableKeyDecoder::NextPrefixEncodingKey(
         if (!s.ok()) {
           return s;
         }
-        if (!file_reader_.file_info_->is_mmap_mode) {
+        if (!file_reader_.file_info()->is_mmap_mode) {
           // In non-mmap mode, we need to make a copy of keys returned to
           // users, because after reading value for the key, the key might
           // be invalid.
@@ -406,16 +424,16 @@ Status PlainTableKeyDecoder::NextPrefixEncodingKey(
           cur_key_.Reserve(prefix_len_ + size);
           cur_key_.SetInternalKey(tmp, *parsed_key);
           parsed_key->user_key =
-              Slice(cur_key_.GetKey().data(), prefix_len_ + size);
-          saved_user_key_ = cur_key_.GetKey();
+              Slice(cur_key_.GetInternalKey().data(), prefix_len_ + size);
+          saved_user_key_ = cur_key_.GetUserKey();
         } else {
           cur_key_.Reserve(prefix_len_ + size);
           cur_key_.SetInternalKey(Slice(saved_user_key_.data(), prefix_len_),
                                   *parsed_key);
         }
-        parsed_key->user_key = ExtractUserKey(cur_key_.GetKey());
+        parsed_key->user_key = cur_key_.GetUserKey();
         if (internal_key != nullptr) {
-          *internal_key = cur_key_.GetKey();
+          *internal_key = cur_key_.GetInternalKey();
         }
         break;
       }
