@@ -1,7 +1,7 @@
-//  Copyright (c) 2013, Facebook, Inc.  All rights reserved.
-//  This source code is licensed under the BSD-style license found in the
-//  LICENSE file in the root directory of this source tree. An additional grant
-//  of patent rights can be found in the PATENTS file in the same directory.
+//  Copyright (c) 2011-present, Facebook, Inc.  All rights reserved.
+//  This source code is licensed under both the GPLv2 (found in the
+//  COPYING file in the root directory) and Apache 2.0 License
+//  (found in the LICENSE.Apache file in the root directory).
 //
 #ifndef ROCKSDB_LITE
 
@@ -12,19 +12,22 @@
 #endif
 
 #include <inttypes.h>
+#include <iostream>
 #include <map>
+#include <memory>
 #include <sstream>
 #include <vector>
 
 #include "db/memtable.h"
 #include "db/write_batch_internal.h"
+#include "options/cf_options.h"
 #include "rocksdb/db.h"
 #include "rocksdb/env.h"
-#include "rocksdb/immutable_options.h"
 #include "rocksdb/iterator.h"
 #include "rocksdb/slice_transform.h"
 #include "rocksdb/status.h"
 #include "rocksdb/table_properties.h"
+#include "rocksdb/utilities/ldb_cmd.h"
 #include "table/block.h"
 #include "table/block_based_table_builder.h"
 #include "table/block_based_table_factory.h"
@@ -32,15 +35,14 @@
 #include "table/format.h"
 #include "table/meta_blocks.h"
 #include "table/plain_table_factory.h"
-#include "tools/ldb_cmd.h"
+#include "table/table_reader.h"
+#include "util/compression.h"
 #include "util/random.h"
 
 #include "port/port.h"
 
 namespace rocksdb {
 
-using std::dynamic_pointer_cast;
-
 SstFileReader::SstFileReader(const std::string& file_path,
                              bool verify_checksum,
                              bool output_hex)
@@ -58,8 +60,22 @@ extern const uint64_t kLegacyPlainTableMagicNumber;
 
 const char* testFileName = "test_file_name";
 
+static const std::vector<std::pair<CompressionType, const char*>>
+    kCompressions = {
+        {CompressionType::kNoCompression, "kNoCompression"},
+        {CompressionType::kSnappyCompression, "kSnappyCompression"},
+        {CompressionType::kZlibCompression, "kZlibCompression"},
+        {CompressionType::kBZip2Compression, "kBZip2Compression"},
+        {CompressionType::kLZ4Compression, "kLZ4Compression"},
+        {CompressionType::kLZ4HCCompression, "kLZ4HCCompression"},
+        {CompressionType::kXpressCompression, "kXpressCompression"},
+        {CompressionType::kZSTD, "kZSTD"}};
+
 Status SstFileReader::GetTableReader(const std::string& file_path) {
-  uint64_t magic_number;
+  // Warning about 'magic_number' being uninitialized shows up only in UBsan
+  // builds. Though access is guarded by 's.ok()' checks, fix the issue to
+  // avoid any warnings.
+  uint64_t magic_number = Footer::kInvalidTableMagicNumber;
 
   // read table magic number
   Footer footer;
@@ -71,10 +87,11 @@ Status SstFileReader::GetTableReader(const std::string& file_path) {
     s = options_.env->GetFileSize(file_path, &file_size);
   }
 
-  file_.reset(new RandomAccessFileReader(std::move(file)));
+  file_.reset(new RandomAccessFileReader(std::move(file), file_path));
 
   if (s.ok()) {
-    s = ReadFooterFromFile(file_.get(), file_size, &footer);
+    s = ReadFooterFromFile(file_.get(), nullptr /* prefetch_buffer */,
+                           file_size, &footer);
   }
   if (s.ok()) {
     magic_number = footer.table_magic_number();
@@ -85,7 +102,7 @@ Status SstFileReader::GetTableReader(const std::string& file_path) {
         magic_number == kLegacyPlainTableMagicNumber) {
       soptions_.use_mmap_reads = true;
       options_.env->NewRandomAccessFile(file_path, &file, soptions_);
-      file_.reset(new RandomAccessFileReader(std::move(file)));
+      file_.reset(new RandomAccessFileReader(std::move(file), file_path));
     }
     options_.comparator = &internal_comparator_;
     // For old sst format, ReadTableProperties might fail but file can be read
@@ -109,24 +126,23 @@ Status SstFileReader::NewTableReader(
     unique_ptr<TableReader>* table_reader) {
   // We need to turn off pre-fetching of index and filter nodes for
   // BlockBasedTable
-  shared_ptr<BlockBasedTableFactory> block_table_factory =
-      dynamic_pointer_cast<BlockBasedTableFactory>(options_.table_factory);
-
-  if (block_table_factory) {
-    return block_table_factory->NewTableReader(
+  if (BlockBasedTableFactory::kName == options_.table_factory->Name()) {
+    return options_.table_factory->NewTableReader(
         TableReaderOptions(ioptions_, soptions_, internal_comparator_,
                            /*skip_filters=*/false),
         std::move(file_), file_size, &table_reader_, /*enable_prefetch=*/false);
   }
 
-  assert(!block_table_factory);
-
   // For all other factory implementation
   return options_.table_factory->NewTableReader(
       TableReaderOptions(ioptions_, soptions_, internal_comparator_),
       std::move(file_), file_size, &table_reader_);
 }
 
+Status SstFileReader::VerifyChecksum() {
+  return table_reader_->VerifyChecksum();
+}
+
 Status SstFileReader::DumpTable(const std::string& out_filename) {
   unique_ptr<WritableFile> out_file;
   Env* env = Env::Default();
@@ -169,7 +185,10 @@ uint64_t SstFileReader::CalculateCompressedTableSize(
   return size;
 }
 
-int SstFileReader::ShowAllCompressionSizes(size_t block_size) {
+int SstFileReader::ShowAllCompressionSizes(
+    size_t block_size,
+    const std::vector<std::pair<CompressionType, const char*>>&
+        compression_types) {
   ReadOptions read_options;
   Options opts;
   const ImmutableCFOptions imoptions(opts);
@@ -177,34 +196,24 @@ int SstFileReader::ShowAllCompressionSizes(size_t block_size) {
   std::vector<std::unique_ptr<IntTblPropCollectorFactory> >
       block_based_table_factories;
 
-  std::map<CompressionType, const char*> compress_type;
-  compress_type.insert(
-      std::make_pair(CompressionType::kNoCompression, "kNoCompression"));
-  compress_type.insert(std::make_pair(CompressionType::kSnappyCompression,
-                                      "kSnappyCompression"));
-  compress_type.insert(
-      std::make_pair(CompressionType::kZlibCompression, "kZlibCompression"));
-  compress_type.insert(
-      std::make_pair(CompressionType::kBZip2Compression, "kBZip2Compression"));
-  compress_type.insert(
-      std::make_pair(CompressionType::kLZ4Compression, "kLZ4Compression"));
-  compress_type.insert(
-      std::make_pair(CompressionType::kLZ4HCCompression, "kLZ4HCCompression"));
-  compress_type.insert(std::make_pair(CompressionType::kZSTDNotFinalCompression,
-                                      "kZSTDNotFinalCompression"));
-
   fprintf(stdout, "Block Size: %" ROCKSDB_PRIszt "\n", block_size);
 
-  for (CompressionType i = CompressionType::kNoCompression;
-       i <= CompressionType::kZSTDNotFinalCompression;
-       i = (i == kLZ4HCCompression) ? kZSTDNotFinalCompression
-                                    : CompressionType(i + 1)) {
-    CompressionOptions compress_opt;
-    TableBuilderOptions tb_opts(imoptions, ikc, &block_based_table_factories, i,
-                                compress_opt, false);
-    uint64_t file_size = CalculateCompressedTableSize(tb_opts, block_size);
-    fprintf(stdout, "Compression: %s", compress_type.find(i)->second);
-    fprintf(stdout, " Size: %" PRIu64 "\n", file_size);
+  for (auto& i : compression_types) {
+    if (CompressionTypeSupported(i.first)) {
+      CompressionOptions compress_opt;
+      std::string column_family_name;
+      int unknown_level = -1;
+      TableBuilderOptions tb_opts(imoptions, ikc, &block_based_table_factories,
+                                  i.first, compress_opt,
+                                  nullptr /* compression_dict */,
+                                  false /* skip_filters */, column_family_name,
+                                  unknown_level);
+      uint64_t file_size = CalculateCompressedTableSize(tb_opts, block_size);
+      fprintf(stdout, "Compression: %s", i.second);
+      fprintf(stdout, " Size: %" PRIu64 "\n", file_size);
+    } else {
+      fprintf(stdout, "Unsupported compression type: %s.\n", i.second);
+    }
   }
   return 0;
 }
@@ -214,8 +223,7 @@ Status SstFileReader::ReadTableProperties(uint64_t table_magic_number,
                                           uint64_t file_size) {
   TableProperties* table_properties = nullptr;
   Status s = rocksdb::ReadTableProperties(file, file_size, table_magic_number,
-                                          options_.env, options_.info_log.get(),
-                                          &table_properties);
+                                          ioptions_, &table_properties);
   if (s.ok()) {
     table_properties_.reset(table_properties);
   } else {
@@ -275,12 +283,10 @@ Status SstFileReader::SetOldTableOptions() {
   return Status::OK();
 }
 
-Status SstFileReader::ReadSequential(bool print_kv,
-                                     uint64_t read_num,
-                                     bool has_from,
-                                     const std::string& from_key,
-                                     bool has_to,
-                                     const std::string& to_key) {
+Status SstFileReader::ReadSequential(bool print_kv, uint64_t read_num,
+                                     bool has_from, const std::string& from_key,
+                                     bool has_to, const std::string& to_key,
+                                     bool use_from_as_prefix) {
   if (!table_reader_) {
     return init_result_;
   }
@@ -310,6 +316,11 @@ Status SstFileReader::ReadSequential(bool print_kv,
       continue;
     }
 
+    // the key returned is not prefixed with out 'from' key
+    if (use_from_as_prefix && !ikey.user_key.starts_with(from_key)) {
+      break;
+    }
+
     // If end marker was specified, we stop before it
     if (has_to && BytewiseComparator()->Compare(ikey.user_key, to_key) >= 0) {
       break;
@@ -343,16 +354,57 @@ namespace {
 
 void print_help() {
   fprintf(stderr,
-          "sst_dump [--command=check|scan|none|raw] [--verify_checksum] "
-          "--file=data_dir_OR_sst_file"
-          " [--output_hex]"
-          " [--input_key_hex]"
-          " [--from=<user_key>]"
-          " [--to=<user_key>]"
-          " [--read_num=NUM]"
-          " [--show_properties]"
-          " [--show_compression_sizes]"
-          " [--show_compression_sizes [--set_block_size=<block_size>]]\n");
+          R"(sst_dump --file=<data_dir_OR_sst_file> [--command=check|scan|raw]
+    --file=<data_dir_OR_sst_file>
+      Path to SST file or directory containing SST files
+
+    --command=check|scan|raw|verify
+        check: Iterate over entries in files but dont print anything except if an error is encounterd (default command)
+        scan: Iterate over entries in files and print them to screen
+        raw: Dump all the table contents to <file_name>_dump.txt
+        verify: Iterate all the blocks in files verifying checksum to detect possible coruption but dont print anything except if a corruption is encountered
+        recompress: reports the SST file size if recompressed with different
+                    compression types
+
+    --output_hex
+      Can be combined with scan command to print the keys and values in Hex
+
+    --from=<user_key>
+      Key to start reading from when executing check|scan
+
+    --to=<user_key>
+      Key to stop reading at when executing check|scan
+
+    --prefix=<user_key>
+      Returns all keys with this prefix when executing check|scan
+      Cannot be used in conjunction with --from
+
+    --read_num=<num>
+      Maximum number of entries to read when executing check|scan
+
+    --verify_checksum
+      Verify file checksum when executing check|scan
+
+    --input_key_hex
+      Can be combined with --from and --to to indicate that these values are encoded in Hex
+
+    --show_properties
+      Print table properties after iterating over the file when executing
+      check|scan|raw
+
+    --set_block_size=<block_size>
+      Can be combined with --command=recompress to set the block size that will
+      be used when trying different compression algorithms
+
+    --compression_types=<comma-separated list of CompressionType members, e.g.,
+      kSnappyCompression>
+      Can be combined with --command=recompress to run recompression for this
+      list of compression types
+
+    --parse_internal_key=<0xKEY>
+      Convenience option to parse an internal key on the command line. Dumps the
+      internal key in hex format {'key' @ SN: type}
+)");
 }
 
 }  // namespace
@@ -369,13 +421,20 @@ int SSTDumpTool::Run(int argc, char** argv) {
   bool input_key_hex = false;
   bool has_from = false;
   bool has_to = false;
+  bool use_from_as_prefix = false;
   bool show_properties = false;
-  bool show_compression_sizes = false;
+  bool show_summary = false;
   bool set_block_size = false;
   std::string from_key;
   std::string to_key;
   std::string block_size_str;
   size_t block_size;
+  std::vector<std::pair<CompressionType, const char*>> compression_types;
+  uint64_t total_num_files = 0;
+  uint64_t total_num_data_blocks = 0;
+  uint64_t total_data_block_size = 0;
+  uint64_t total_index_block_size = 0;
+  uint64_t total_filter_block_size = 0;
   for (int i = 1; i < argc; i++) {
     if (strncmp(argv[i], "--file=", 7) == 0) {
       dir_or_file = argv[i] + 7;
@@ -397,27 +456,73 @@ int SSTDumpTool::Run(int argc, char** argv) {
     } else if (strncmp(argv[i], "--to=", 5) == 0) {
       to_key = argv[i] + 5;
       has_to = true;
+    } else if (strncmp(argv[i], "--prefix=", 9) == 0) {
+      from_key = argv[i] + 9;
+      use_from_as_prefix = true;
     } else if (strcmp(argv[i], "--show_properties") == 0) {
       show_properties = true;
-    } else if (strcmp(argv[i], "--show_compression_sizes") == 0) {
-      show_compression_sizes = true;
+    } else if (strcmp(argv[i], "--show_summary") == 0) {
+      show_summary = true;
     } else if (strncmp(argv[i], "--set_block_size=", 17) == 0) {
       set_block_size = true;
       block_size_str = argv[i] + 17;
       std::istringstream iss(block_size_str);
+      iss >> block_size;
       if (iss.fail()) {
-        fprintf(stderr, "block size must be numeric");
+        fprintf(stderr, "block size must be numeric\n");
         exit(1);
       }
-      iss >> block_size;
+    } else if (strncmp(argv[i], "--compression_types=", 20) == 0) {
+      std::string compression_types_csv = argv[i] + 20;
+      std::istringstream iss(compression_types_csv);
+      std::string compression_type;
+      while (std::getline(iss, compression_type, ',')) {
+        auto iter = std::find_if(
+            kCompressions.begin(), kCompressions.end(),
+            [&compression_type](std::pair<CompressionType, const char*> curr) {
+              return curr.second == compression_type;
+            });
+        if (iter == kCompressions.end()) {
+          fprintf(stderr, "%s is not a valid CompressionType\n",
+                  compression_type.c_str());
+          exit(1);
+        }
+        compression_types.emplace_back(*iter);
+      }
+    } else if (strncmp(argv[i], "--parse_internal_key=", 21) == 0) {
+      std::string in_key(argv[i] + 21);
+      try {
+        in_key = rocksdb::LDBCommand::HexToString(in_key);
+      } catch (...) {
+        std::cerr << "ERROR: Invalid key input '"
+          << in_key
+          << "' Use 0x{hex representation of internal rocksdb key}" << std::endl;
+        return -1;
+      }
+      Slice sl_key = rocksdb::Slice(in_key);
+      ParsedInternalKey ikey;
+      int retc = 0;
+      if (!ParseInternalKey(sl_key, &ikey)) {
+        std::cerr << "Internal Key [" << sl_key.ToString(true /* in hex*/)
+                  << "] parse error!\n";
+        retc = -1;
+      }
+      fprintf(stdout, "key=%s\n", ikey.DebugString(true).c_str());
+      return retc;
     } else {
+      fprintf(stderr, "Unrecognized argument '%s'\n\n", argv[i]);
       print_help();
       exit(1);
     }
   }
 
+  if (use_from_as_prefix && has_from) {
+    fprintf(stderr, "Cannot specify --prefix and --from\n\n");
+    exit(1);
+  }
+
   if (input_key_hex) {
-    if (has_from) {
+    if (has_from || use_from_as_prefix) {
       from_key = rocksdb::LDBCommand::HexToString(from_key);
     }
     if (has_to) {
@@ -426,6 +531,7 @@ int SSTDumpTool::Run(int argc, char** argv) {
   }
 
   if (dir_or_file == nullptr) {
+    fprintf(stderr, "file or directory must be specified.\n\n");
     print_help();
     exit(1);
   }
@@ -461,15 +567,13 @@ int SSTDumpTool::Run(int argc, char** argv) {
     if (!reader.getStatus().ok()) {
       fprintf(stderr, "%s: %s\n", filename.c_str(),
               reader.getStatus().ToString().c_str());
-      exit(1);
+      continue;
     }
 
-    if (show_compression_sizes) {
-      if (set_block_size) {
-        reader.ShowAllCompressionSizes(block_size);
-      } else {
-        reader.ShowAllCompressionSizes(16384);
-      }
+    if (command == "recompress") {
+      reader.ShowAllCompressionSizes(
+          set_block_size ? block_size : 16384,
+          compression_types.empty() ? kCompressions : compression_types);
       return 0;
     }
 
@@ -489,10 +593,10 @@ int SSTDumpTool::Run(int argc, char** argv) {
 
     // scan all files in give file path.
     if (command == "" || command == "scan" || command == "check") {
-      st = reader.ReadSequential(command == "scan",
-                                 read_num > 0 ? (read_num - total_read) :
-                                                read_num,
-                                 has_from, from_key, has_to, to_key);
+      st = reader.ReadSequential(
+          command == "scan", read_num > 0 ? (read_num - total_read) : read_num,
+          has_from || use_from_as_prefix, from_key, has_to, to_key,
+          use_from_as_prefix);
       if (!st.ok()) {
         fprintf(stderr, "%s: %s\n", filename.c_str(),
             st.ToString().c_str());
@@ -502,7 +606,19 @@ int SSTDumpTool::Run(int argc, char** argv) {
         break;
       }
     }
-    if (show_properties) {
+
+    if (command == "verify") {
+      st = reader.VerifyChecksum();
+      if (!st.ok()) {
+        fprintf(stderr, "%s is corrupted: %s\n", filename.c_str(),
+                st.ToString().c_str());
+      } else {
+        fprintf(stdout, "The file is ok\n");
+      }
+      continue;
+    }
+
+    if (show_properties || show_summary) {
       const rocksdb::TableProperties* table_properties;
 
       std::shared_ptr<const rocksdb::TableProperties>
@@ -516,17 +632,56 @@ int SSTDumpTool::Run(int argc, char** argv) {
         table_properties = table_properties_from_reader.get();
       }
       if (table_properties != nullptr) {
+        if (show_properties) {
+          fprintf(stdout,
+                  "Table Properties:\n"
+                  "------------------------------\n"
+                  "  %s",
+                  table_properties->ToString("\n  ", ": ").c_str());
+          fprintf(stdout, "# deleted keys: %" PRIu64 "\n",
+                  rocksdb::GetDeletedKeys(
+                      table_properties->user_collected_properties));
+
+          bool property_present;
+          uint64_t merge_operands = rocksdb::GetMergeOperands(
+              table_properties->user_collected_properties, &property_present);
+          if (property_present) {
+            fprintf(stdout, "  # merge operands: %" PRIu64 "\n",
+                    merge_operands);
+          } else {
+            fprintf(stdout, "  # merge operands: UNKNOWN\n");
+          }
+        }
+        total_num_files += 1;
+        total_num_data_blocks += table_properties->num_data_blocks;
+        total_data_block_size += table_properties->data_size;
+        total_index_block_size += table_properties->index_size;
+        total_filter_block_size += table_properties->filter_size;
+      }
+      if (show_properties) {
         fprintf(stdout,
-                "Table Properties:\n"
-                "------------------------------\n"
-                "  %s",
-                table_properties->ToString("\n  ", ": ").c_str());
-        fprintf(stdout, "# deleted keys: %" PRIu64 "\n",
-                rocksdb::GetDeletedKeys(
-                    table_properties->user_collected_properties));
+                "Raw user collected properties\n"
+                "------------------------------\n");
+        for (const auto& kv : table_properties->user_collected_properties) {
+          std::string prop_name = kv.first;
+          std::string prop_val = Slice(kv.second).ToString(true);
+          fprintf(stdout, "  # %s: 0x%s\n", prop_name.c_str(),
+                  prop_val.c_str());
+        }
       }
     }
   }
+  if (show_summary) {
+    fprintf(stdout, "total number of files: %" PRIu64 "\n", total_num_files);
+    fprintf(stdout, "total number of data blocks: %" PRIu64 "\n",
+            total_num_data_blocks);
+    fprintf(stdout, "total data block size: %" PRIu64 "\n",
+            total_data_block_size);
+    fprintf(stdout, "total index block size: %" PRIu64 "\n",
+            total_index_block_size);
+    fprintf(stdout, "total filter block size: %" PRIu64 "\n",
+            total_filter_block_size);
+  }
   return 0;
 }
 }  // namespace rocksdb
