@@ -1,7 +1,7 @@
-//  Copyright (c) 2013, Facebook, Inc.  All rights reserved.
-//  This source code is licensed under the BSD-style license found in the
-//  LICENSE file in the root directory of this source tree. An additional grant
-//  of patent rights can be found in the PATENTS file in the same directory.
+//  Copyright (c) 2011-present, Facebook, Inc.  All rights reserved.
+//  This source code is licensed under both the GPLv2 (found in the
+//  COPYING file in the root directory) and Apache 2.0 License
+//  (found in the LICENSE.Apache file in the root directory).
 //
 #include "util/file_util.h"
 
@@ -9,7 +9,6 @@
 #include <algorithm>
 
 #include "rocksdb/env.h"
-#include "rocksdb/options.h"
 #include "util/sst_file_manager_impl.h"
 #include "util/file_reader_writer.h"
 
@@ -17,7 +16,7 @@ namespace rocksdb {
 
 // Utility function to copy a file up to a specified length
 Status CopyFile(Env* env, const std::string& source,
-                const std::string& destination, uint64_t size) {
+                const std::string& destination, uint64_t size, bool use_fsync) {
   const EnvOptions soptions;
   Status s;
   unique_ptr<SequentialFileReader> src_reader;
@@ -63,12 +62,30 @@ Status CopyFile(Env* env, const std::string& source,
     }
     size -= slice.size();
   }
+  dest_writer->Sync(use_fsync);
   return Status::OK();
 }
 
-Status DeleteSSTFile(const DBOptions* db_options, const std::string& fname,
-                     uint32_t path_id) {
+// Utility function to create a file with the provided contents
+Status CreateFile(Env* env, const std::string& destination,
+                  const std::string& contents) {
+  const EnvOptions soptions;
+  Status s;
+  unique_ptr<WritableFileWriter> dest_writer;
+
+  unique_ptr<WritableFile> destfile;
+  s = env->NewWritableFile(destination, &destfile, soptions);
+  if (!s.ok()) {
+    return s;
+  }
+  dest_writer.reset(new WritableFileWriter(std::move(destfile), soptions));
+  return dest_writer->Append(Slice(contents));
+}
+
+Status DeleteSSTFile(const ImmutableDBOptions* db_options,
+                     const std::string& fname, uint32_t path_id) {
   // TODO(tec): support sst_file_manager for multiple path_ids
+#ifndef ROCKSDB_LITE
   auto sfm =
       static_cast<SstFileManagerImpl*>(db_options->sst_file_manager.get());
   if (sfm && path_id == 0) {
@@ -76,6 +93,10 @@ Status DeleteSSTFile(const DBOptions* db_options, const std::string& fname,
   } else {
     return db_options->env->DeleteFile(fname);
   }
+#else
+  // SstFileManager is not supported in ROCKSDB_LITE
+  return db_options->env->DeleteFile(fname);
+#endif
 }
 
 }  // namespace rocksdb
