@@ -1,9 +1,11 @@
-//  Copyright (c) 2014, Facebook, Inc.  All rights reserved.
-//  This source code is licensed under the BSD-style license found in the
-//  LICENSE file in the root directory of this source tree. An additional grant
-//  of patent rights can be found in the PATENTS file in the same directory.
+//  Copyright (c) 2011-present, Facebook, Inc.  All rights reserved.
+//  This source code is licensed under both the GPLv2 (found in the
+//  COPYING file in the root directory) and Apache 2.0 License
+//  (found in the LICENSE.Apache file in the root directory).
 
 #include "util/sync_point.h"
+#include <functional>
+#include <thread>
 #include "port/port.h"
 #include "util/random.h"
 
@@ -39,7 +41,7 @@ SyncPoint* SyncPoint::GetInstance() {
   return &sync_point;
 }
 
-void SyncPoint::LoadDependency(const std::vector<Dependency>& dependencies) {
+void SyncPoint::LoadDependency(const std::vector<SyncPointPair>& dependencies) {
   std::unique_lock<std::mutex> lock(mutex_);
   successors_.clear();
   predecessors_.clear();
@@ -51,6 +53,27 @@ void SyncPoint::LoadDependency(const std::vector<Dependency>& dependencies) {
   cv_.notify_all();
 }
 
+void SyncPoint::LoadDependencyAndMarkers(
+    const std::vector<SyncPointPair>& dependencies,
+    const std::vector<SyncPointPair>& markers) {
+  std::unique_lock<std::mutex> lock(mutex_);
+  successors_.clear();
+  predecessors_.clear();
+  cleared_points_.clear();
+  markers_.clear();
+  marked_thread_id_.clear();
+  for (const auto& dependency : dependencies) {
+    successors_[dependency.predecessor].push_back(dependency.successor);
+    predecessors_[dependency.successor].push_back(dependency.predecessor);
+  }
+  for (const auto& marker : markers) {
+    successors_[marker.predecessor].push_back(marker.successor);
+    predecessors_[marker.successor].push_back(marker.predecessor);
+    markers_[marker.predecessor].push_back(marker.successor);
+  }
+  cv_.notify_all();
+}
+
 bool SyncPoint::PredecessorsAllCleared(const std::string& point) {
   for (const auto& pred : predecessors_[point]) {
     if (cleared_points_.count(pred) == 0) {
@@ -66,6 +89,14 @@ void SyncPoint::SetCallBack(const std::string point,
   callbacks_[point] = callback;
 }
 
+void SyncPoint::ClearCallBack(const std::string point) {
+  std::unique_lock<std::mutex> lock(mutex_);
+  while (num_callbacks_running_ > 0) {
+    cv_.wait(lock);
+  }
+  callbacks_.erase(point);
+}
+
 void SyncPoint::ClearAllCallBacks() {
   std::unique_lock<std::mutex> lock(mutex_);
   while (num_callbacks_running_ > 0) {
@@ -89,10 +120,38 @@ void SyncPoint::ClearTrace() {
   cleared_points_.clear();
 }
 
+bool SyncPoint::DisabledByMarker(const std::string& point,
+                                 std::thread::id thread_id) {
+  auto marked_point_iter = marked_thread_id_.find(point);
+  return marked_point_iter != marked_thread_id_.end() &&
+         thread_id != marked_point_iter->second;
+}
+
 void SyncPoint::Process(const std::string& point, void* cb_arg) {
   std::unique_lock<std::mutex> lock(mutex_);
+  if (!enabled_) {
+    return;
+  }
 
-  if (!enabled_) return;
+  auto thread_id = std::this_thread::get_id();
+
+  auto marker_iter = markers_.find(point);
+  if (marker_iter != markers_.end()) {
+    for (auto marked_point : marker_iter->second) {
+      marked_thread_id_.insert(std::make_pair(marked_point, thread_id));
+    }
+  }
+
+  if (DisabledByMarker(point, thread_id)) {
+    return;
+  }
+
+  while (!PredecessorsAllCleared(point)) {
+    cv_.wait(lock);
+    if (DisabledByMarker(point, thread_id)) {
+      return;
+    }
+  }
 
   auto callback_pair = callbacks_.find(point);
   if (callback_pair != callbacks_.end()) {
@@ -104,10 +163,6 @@ void SyncPoint::Process(const std::string& point, void* cb_arg) {
     cv_.notify_all();
   }
 
-  while (!PredecessorsAllCleared(point)) {
-    cv_.wait(lock);
-  }
-
   cleared_points_.insert(point);
   cv_.notify_all();
 }
